require 'base64'

module Samsung
  module Protocol
    class Frame
      attr_accessor :data

      def initialize data = ''
        @data = data
      end

      def size
        @data.size
      end

      def hex
        data.split('').map do |c|
          "0x%.2x" % c.ord
        end.join(' ')
      end

      # String

      def push_string(s)
        push_int16(s.size)
        self.data += s
      end

      def pop_string
        len = pop_int16
        data.slice!(0..len-1)
      end

      # Base64 string

      def push_string64(s)
        push_string(Base64.encode64(s))
      end

      def pop_string64
        len = pop_int16
        Base64.decode64(data.slice!(0..len-1))
      end

      # 8-bit integer

      def push_int8(v)
        self.data += [v].pack('C')
      end

      def pop_int8
        data.slice!(0).unpack('C').first
      end

      # 16-bit integer

      def push_int16(v)
        self.data += [v].pack('S<')
      end

      def pop_int16
        data.slice!(0..1).unpack('S<').first
      end

      # 32-bit integer

      def push_int32(v)
        self.data += [v].pack('L<')
      end

      def pop_int32
        data.slice!(0..1).unpack('L<').first
      end

      # Frames

      def push_frame(frame)
        push_int16(frame.size)
        self.data += frame.data
      end

      def pop_frame
        frame_size = pop_int16
        Frame.new(data.slice!(0..frame_size-1))
      end
    end
  end
end
