require 'socket'

module Samsung
  class Remote
    PORT = 55000
    APP_NAME = "net.kapati.samsung.remote"

    attr_accessor :display_name, :ip

    def initialize(ip, display_name = "Ruby")
      self.ip = ip
      self.display_name = display_name
      connect
      send_auth
    end

    def has_data?(wait = 0)
      ready = IO.select([@socket], nil, nil, wait)
      return ready != nil
    end

    def peek_frame(wait = 1)
      return nil unless has_data?(wait)
      read_frame
    end

    def read_frame
      data = @socket.readpartial(512)
      Protocol::Response.new(data)
    end

    def connect
      @socket = TCPSocket.open ip, PORT
    end

    def send_frame(frame)
      f = build_header
      f.push_frame(frame)
      @socket.write(f.data)
    end

    def send_key key_code
      send_frame(Protocol::KeyFrame.new(key_code))
    end

    def send_auth
      net = Network.new
      send_frame(
        Protocol::AuthFrame.new(
          net.local_ip,
          net.local_hw_address,
          display_name
        )
      )
    end

    private

    def build_header
      Protocol::HeaderFrame.new(APP_NAME)
    end
  end
end
